@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import numpy as np
-from .utils import WARN, EPS, p2up, adm_ssq, process_scales, _process_fs_and_t
-from .ssqueezing import ssqueeze, phase_cwt, phase_cwt_num
+from .utils import WARN, EPS, pi, p2up, adm_ssq, process_scales, _process_fs_and_t
+from .ssqueezing import ssqueeze, _check_squeezing_arg
 from .wavelets import Wavelet
 from ._cwt import cwt
 
@@ -9,7 +9,7 @@ from ._cwt import cwt
 def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
             ssq_freqs=None, padtype='reflect', squeezing='sum', mapkind='maximal',
             difftype='direct', difforder=None, gamma=None):
-    """Calculates the synchrosqueezed Continuous Wavelet Transform of `x`.
+    """Synchrosqueezed Continuous Wavelet Transform.
     Implements the algorithm described in Sec. III of [1].
 
     # Arguments:
@@ -17,10 +17,10 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
             Input vector, 1D.
 
         wavelet: str / tuple[str, dict] / `wavelets.Wavelet`
-            Wavelet sampled in Fourier frequency domain. See help(cwt).
+            Wavelet sampled in Fourier frequency domain. See `help(cwt)`.
 
         scales: str['log', 'linear', 'log:maximal', ...] / np.ndarray
-            CWT scales. See help(cwt).
+            CWT scales. See `help(cwt)`.
 
         nv: int / None
             Number of voices (CWT only). Suggested >= 32 (default=32).
@@ -80,12 +80,12 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
             instantaneous frequencies:
                     w(a,b) = Im( (1/2pi) * (1/Wx(a,b)) * d/db[Wx(a,b)] )
 
-                - 'direct': use `dWx`, obtained via frequency-domain
-                differentiation (see `cwt`, `phase_cwt`).
-                - 'phase': differentiate by taking forward finite-difference of
-                unwrapped angle of `Wx` (see `phase_cwt`).
-                - 'numeric': first-, second-, or fourth-order (set by
-                `difforder`) numeric differentiation (see `phase_cwt_num`).
+            - 'direct': use `dWx`, obtained via frequency-domain differentiation
+            (see `cwt`, `phase_cwt`).
+            - 'phase': differentiate by taking forward finite-difference of
+            unwrapped angle of `Wx` (see `phase_cwt`).
+            - 'numeric': first-, second-, or fourth-order (set by `difforder`)
+            numeric differentiation (see `phase_cwt_num`).
 
         difforder: int[1, 2, 4]
             Order of differentiation for difftype='numeric' (default=4).
@@ -146,9 +146,9 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
                                  "(got %s)" % difforder)
         elif difftype == 'numeric':
             difforder = 4
-        if squeezing not in ('sum', 'lebesgue'):
-            raise ValueError("`squeezing` must be one of: sum, lebesgue "
-                             "(got %s)" % squeezing)
+
+        _check_squeezing_arg(squeezing)
+
         if nv is None and not isinstance(scales, np.ndarray):
             nv = 32
 
@@ -165,7 +165,7 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
             # calculate inst. freq. from unwrapped phase of CWT
             w = phase_cwt(Wx, None, difftype, gamma)
         elif difftype == 'numeric':
-            # !!! tested to be very inaccurate for small `a`
+            # !!! tested to be very inaccurate for small scales
             # calculate derivative numericly
             _, n1, _ = p2up(N)
             Wx = Wx[:, (n1 - 4):(n1 + N + 4)]
@@ -180,12 +180,11 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
     scales, cwt_scaletype, *_ = process_scales(scales, N, wavelet, nv=nv,
                                                get_params=True)
 
-    # l1_norm=True to spare a multiplication; for SSWT L1 & L2 are exactly same
-    # anyway since we're inverting CWT over time-frequency plane
+    # l1_norm=True to spare a multiplication; for SSWT L1 & L2 are exactly
+    # same anyway since we're inverting CWT over time-frequency plane
     rpadded = (difftype == 'numeric')
-    Wx, scales, _, dWx = cwt(x, wavelet, scales=scales, fs=fs, nv=nv,
-                             l1_norm=True, derivative=True, padtype=padtype,
-                             rpadded=rpadded)
+    Wx, scales, dWx = cwt(x, wavelet, scales=scales, fs=fs, nv=nv, l1_norm=True,
+                          derivative=True, padtype=padtype, rpadded=rpadded)
 
     gamma = gamma or np.sqrt(EPS)
     Wx, w = _phase_transform(Wx, dWx, N, dt, gamma, difftype, difforder)
@@ -202,7 +201,7 @@ def ssq_cwt(x, wavelet='morlet', scales='log', nv=None, fs=None, t=None,
         Wx = Wx[:, 4:-4]
         w  = w[:,  4:-4]
         Tx = Tx[:, 4:-4]
-    return Tx, ssq_freqs, Wx, scales, w
+    return Tx, ssq_freqs, Wx, scales, w  # TODO change return order
 
 
 def issq_cwt(Tx, wavelet, cc=None, cw=None):
@@ -258,42 +257,7 @@ def issq_cwt(Tx, wavelet, cc=None, cw=None):
         https://github.com/ebrevdo/synchrosqueezing/blob/master/synchrosqueezing/
         synsq_cwt_iw.m
     """
-    def _invert_components(Tx, cc, cw):
-        # Invert Tx around curve masks in the time-frequency plane to recover
-        # individual components; last one is the remaining signal
-        x = np.zeros((cc.shape[1] + 1, cc.shape[0]))
-        TxRemainder = Tx.copy()
-
-        for n in range(cc.shape[1]):
-            TxMask = np.zeros(Tx.shape, dtype='complex128')
-            upper_cc = np.clip(cc[:, n] + cw[:, n], 0, len(Tx))
-            lower_cc = np.clip(cc[:, n] - cw[:, n], 0, len(Tx))
-
-            # cc==-1 denotes no curve at that time,
-            # removing such points from inversion
-            upper_cc[np.where(cc[:, n] == -1)] = 0
-            lower_cc[np.where(cc[:, n] == -1)] = 1
-            for m in range(Tx.shape[1]):
-                idxs = slice(lower_cc[m], upper_cc[m] + 1)
-                TxMask[idxs, m] = Tx[idxs, m]
-                TxRemainder[idxs, m] = 0
-            x[n] = TxMask.real.sum(axis=0).T
-
-        x[n + 1] = TxRemainder.real.sum(axis=0).T
-        return x
-
-    def _process_args(cc, cw):
-        if (cc is None) and (cw is None):
-            return None, None, True
-        if cc.ndim == 1:
-            cc = cc.reshape(-1, 1)
-        if cw.ndim == 1:
-            cw = cw.reshape(-1, 1)
-        cc = cc.astype('int32')
-        cw = cw.astype('int32')
-        return cc, cw, False
-
-    cc, cw, full_inverse = _process_args(cc, cw)
+    cc, cw, full_inverse = _process_component_inversion_args(cc, cw)
 
     if full_inverse:
         # Integration over all frequencies recovers original signal
@@ -306,3 +270,196 @@ def issq_cwt(Tx, wavelet, cc=None, cw=None):
     # *2 per analytic wavelet & taking real part; Theorem 4.5 [2]
     x *= (2 / Css)
     return x
+
+
+def _invert_components(Tx, cc, cw):
+    # Invert Tx around curve masks in the time-frequency plane to recover
+    # individual components; last one is the remaining signal
+    x = np.zeros((cc.shape[1] + 1, cc.shape[0]))
+    TxRemainder = Tx.copy()
+
+    for n in range(cc.shape[1]):
+        TxMask = np.zeros(Tx.shape, dtype='complex128')
+        upper_cc = np.clip(cc[:, n] + cw[:, n], 0, len(Tx))
+        lower_cc = np.clip(cc[:, n] - cw[:, n], 0, len(Tx))
+
+        # cc==-1 denotes no curve at that time,
+        # removing such points from inversion
+        upper_cc[np.where(cc[:, n] == -1)] = 0
+        lower_cc[np.where(cc[:, n] == -1)] = 1
+        for m in range(Tx.shape[1]):
+            idxs = slice(lower_cc[m], upper_cc[m] + 1)
+            TxMask[idxs, m] = Tx[idxs, m]
+            TxRemainder[idxs, m] = 0
+        x[n] = TxMask.real.sum(axis=0).T
+
+    x[n + 1] = TxRemainder.real.sum(axis=0).T
+    return x
+
+
+def _process_component_inversion_args(cc, cw):
+    if (cc is None) and (cw is None):
+        full_inverse = True
+    else:
+        full_inverse = False
+        if cc.ndim == 1:
+            cc = cc.reshape(-1, 1)
+        if cw.ndim == 1:
+            cw = cw.reshape(-1, 1)
+        cc = cc.astype('int32')
+        cw = cw.astype('int32')
+    return cc, cw, full_inverse
+
+
+def phase_cwt(Wx, dWx, difftype='direct', gamma=None):
+    """Calculate the phase transform at each (scale, time) pair:
+          w[a, b] = Im((1/2pi) * d/db (Wx[a,b]) / Wx[a,b])
+    See above Eq 20.3 in [1], or Eq 13 in [2].
+
+    # Arguments:
+        Wx: np.ndarray
+            CWT of `x` (see `cwt`).
+
+        dWx: np.ndarray.
+            Time-derivative of `Wx`, computed via frequency-domain differentiation
+            (effectively, derivative of trigonometric interpolation; see [4]).
+
+        difftype: str['direct', 'phase']
+            Method by which to differentiate Wx (default='direct') to obtain
+            instantaneous frequencies:
+                    w(a,b) = Im( (1/2pi) * (1/Wx(a,b)) * d/db[Wx(a,b)] )
+
+                - 'direct': using `dWx` (see `dWx`).
+                - 'phase': differentiate by taking forward finite-difference of
+                unwrapped angle of `Wx` (see `phase_cwt`).
+
+        gamma: float / None
+            CWT phase threshold. Sets `w=inf` for small values of `Wx` where
+            phase computation is unstable and inaccurate (like in DFT):
+                w[abs(Wx) < beta] = inf
+            This is used to zero `Wx` where `w=0` in computing `Tx` to ignore
+            contributions from points with indeterminate phase.
+            Default = sqrt(machine epsilon) = np.sqrt(np.finfo(np.float64).eps)
+
+    # Returns:
+        w: np.ndarray
+            Phase transform for each element of `Wx`. w.shape == Wx.shape.
+
+    # References:
+        1. A Nonlinear squeezing of the CWT Based on Auditory Nerve Models.
+        I. Daubechies, S. Maes.
+        https://services.math.duke.edu/%7Eingrid/publications/DM96.pdf
+
+        2. The Synchrosqueezing algorithm for time-varying spectral analysis:
+        robustness properties and new paleoclimate applications.
+        G. Thakur, E. Brevdo, N.-S. Fučkar, and H.-T. Wu.
+        https://arxiv.org/abs/1105.0010
+
+        3. Synchrosqueezed Wavelet Transforms: a Tool for Empirical Mode
+        Decomposition. I. Daubechies, J. Lu, H.T. Wu.
+        https://arxiv.org/pdf/0912.2437.pdf
+
+        4. The Exponential Accuracy of Fourier and Chebyshev Differencing Methods.
+        E. Tadmor.
+        http://webhome.auburn.edu/~jzl0097/teaching/math_8970/Tadmor_86.pdf
+
+        5. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
+        https://github.com/ebrevdo/synchrosqueezing/blob/master/synchrosqueezing/
+        phase_cwt.m
+    """
+    if difftype == 'phase':
+        # TODO gives bad results; shouldn't we divide by Wx?
+        u = np.unwrap(np.angle(Wx)).T
+        w = np.vstack([np.diff(u, axis=0), u[-1] - u[0]]).T / (2*pi)
+    else:
+        with np.errstate(divide='ignore'):
+            w = np.imag(dWx / Wx) / (2*pi)
+    # treat negative phases as positive; these are in small minority, and
+    # slightly aid invertibility (as less of `Wx` is zeroed in ssqueezing)
+    w = np.abs(w)
+
+    gamma = gamma or np.sqrt(EPS)
+    w[np.abs(Wx) < gamma] = np.inf
+    return w
+
+
+def phase_cwt_num(Wx, dt, difforder=4, gamma=None):
+    """Calculate the phase transform at each (scale, time) pair:
+        w[a, b] = Im((1/2pi) * d/db (Wx[a,b]) / Wx[a,b])
+    Uses numeric differentiation (1st, 2nd, or 4th order). See above Eq 20.3
+    in [1], or Eq 13 in [2].
+
+    # Arguments:
+        Wx: np.ndarray. Wavelet transform of `x` (see `cwt`).
+
+        dt: int. Sampling period (e.g. t[1] - t[0]).
+
+        difforder: int[1, 2, 4]
+            Order of differentiation (default=4).
+
+        gamma: float
+            CWT phase threshold. Sets `w=inf` for small values of `Wx` where
+            phase computation is unstable and inaccurate (like in DFT):
+                w[abs(Wx) < beta] = inf
+            This is used to zero `Wx` where `w=0` in computing `Tx` to ignore
+            contributions from points with indeterminate phase.
+            Default = sqrt(machine epsilon) = np.sqrt(np.finfo(np.float64).eps)
+
+    # Returns:
+        w: np.ndarray
+            Phase transform via demodulated FM-estimates. w.shape == Wx.shape.
+
+    # References:
+        1. A Nonlinear squeezing of the CWT Based on Auditory Nerve Models.
+        I. Daubechies, S. Maes.
+        https://services.math.duke.edu/%7Eingrid/publications/DM96.pdf
+
+        2. The Synchrosqueezing algorithm for time-varying spectral analysis:
+        robustness properties and new paleoclimate applications.
+        G. Thakur, E. Brevdo, N.-S. Fučkar, and H.-T. Wu.
+        https://arxiv.org/abs/1105.0010
+
+        3. Synchrosqueezing Toolbox, (C) 2014--present. E. Brevdo, G. Thakur.
+        https://github.com/ebrevdo/synchrosqueezing/blob/master/synchrosqueezing/
+        phase_cwt_num.m
+    """
+    # unreliable; bad results on high freq pure tones
+    def _differentiate(Wx, dt):
+        if difforder in (2, 4):
+            # append for differentiating
+            Wxr = np.hstack([Wx[:, -2:], Wx, Wx[:, :2]])
+
+        if difforder == 1:
+            w = np.hstack([Wx[:, 1:] - Wx[:, :-1],
+                           Wx[:, :1]  - Wx[:, -1:]])
+            w /= dt
+        elif difforder == 2:
+            # calculate 2nd-order forward difference
+            w = -Wxr[:, 4:] + 4 * Wxr[:, 3:-1] - 3 * Wxr[:, 2:-2]
+            w /= (2 * dt)
+        elif difforder == 4:
+            # calculate 4th-order central difference
+            w = -Wxr[:, 4:]
+            w += Wxr[:, 3:-1] * 8
+            w -= Wxr[:, 1:-3] * 8
+            w += Wxr[:, 0:-4]
+            w /= (12 * dt)
+        return w
+
+    # epsilon from Daubechies, H-T Wu, et al.
+    # gamma from Brevdo, H-T Wu, et al.
+    gamma = gamma or np.sqrt(EPS)
+    if difforder not in (1, 2, 4):
+        raise ValueError("`difforder` must be one of: 1, 2, 4 "
+                         "(got %s)" % difforder)
+
+    w = _differentiate(Wx, dt)
+    w[np.abs(Wx) < gamma] = np.inf
+
+    # calculate inst. freq for each scale
+    # 2*pi norm per discretized inverse FT rather than inverse DFT
+    w = np.real(-1j * w / Wx) / (2*pi)
+
+    # see `phase_cwt`, though negatives may no longer be in minority
+    w = np.abs(w)
+    return w
