@@ -21,12 +21,17 @@ def lin_band(Tx, slope, offset, bw=.025, **kw):
 
 
 #### Signals #################################################################
-def _t(min, max, N):
-    return np.linspace(min, max, N, endpoint=False)
+def _t(min, max, N, endpoint=False):
+    return np.linspace(min, max, N, endpoint=endpoint)
 
-def cos_f(freqs, N=128, phi=0):
+def cos_f(freqs, N=128, phi=0, endpoint=False):
     """Adjacent different frequency cosines."""
-    return np.concatenate([np.cos(2 * np.pi * f * (_t(i, i + 1, N) + phi))
+    return np.concatenate([np.cos(2*np.pi * f * (_t(i, i + 1, N, endpoint) + phi))
+                           for i, f in enumerate(freqs)])
+
+def sin_f(freqs, N=128, phi=0, endpoint=False):
+    """Adjacent different frequency sines."""
+    return np.concatenate([np.sin(2*np.pi * f * (_t(i, i + 1, N, endpoint) + phi))
                            for i, f in enumerate(freqs)])
 
 #### Misc ####################################################################
@@ -34,3 +39,8 @@ def mad_rms(x, xrec):
     """Reconstruction error metric; scale-invariant, robust to outliers
     and partly sparsity. https://stats.stackexchange.com/q/495242/239063"""
     return np.mean(np.abs(x - xrec)) / np.sqrt(np.mean(x**2))
+
+
+def amax(x):
+    """Return N-dimensional indices of where `abs(x) == max(abs(x))`."""
+    return np.where(np.abs(x) == np.abs(x).max())
